LIBRARY rpc

CODE      PRELOAD MOVEABLE DISCARDABLE
DATA      PRELOAD SINGLE

EXPORTS

	authnone_create
	authunix_create
	authunix_create_default
	bcopy
	bzero
	bcmp
	clnt_broadcast
	clnt_create
	clnt_pcreateerror
	clnt_perror
	clnt_sperrno
	clntraw_create
	clnttcp_create
	clntudp_bufcreate
	clntudp_create
	get_myaddress
	getrpcbyname
	getrpcbynumber
	pmap_getmaps
	pmap_getport
	pmap_set
	pmap_unset
	rpc_nt_exit
	rpc_nt_init
	svc_getreq
	svc_getreqset
	svc_register
	svc_run
	svc_sendreply
	svc_unregister
	svcerr_auth
	svcerr_decode
	svcerr_noproc
	svcerr_noprog
	svcerr_progvers
	svcerr_systemerr
	svcerr_weakauth
	svcraw_create
	svctcp_create
	svcudp_bufcreate
	svcudp_create
	xdr_array
	xdr_authunix_parms
	xdr_bool
	xdr_bytes
	xdr_callhdr
	xdr_callmsg
	xdr_char
	xdr_des_block
	xdr_double
	xdr_enum
	xdr_float
	xdr_free
	xdr_int
	xdr_long
	xdr_netobj
	xdr_opaque
	xdr_opaque_auth
	xdr_pmap
	xdr_pmaplist
	xdr_pointer
	xdr_reference
	xdr_replymsg
	xdr_short
	xdr_string
	xdr_u_char
	xdr_u_int
	xdr_u_long
	xdr_u_short
	xdr_union
	xdr_vector
	xdr_void
	xdr_wrapstring
	xdrmem_create
	xdrrec_create
	xdrrec_endofrecord
	xdrrec_eof
	xdrrec_skiprecord
	xdrstdio_create
	xprt_register
	xprt_unregister
