EXETYPE WINDOWS

EXPORTS
    	XML_SetElementDeclHandler
	XML_SetAttlistDeclHandler
	XML_SetXmlDeclHandler
	XML_ParserCreate
	XML_ParserCreateNS
	XML_ParserCreate_MM
	XML_SetEntityDeclHandler
	XML_SetElementHandler
	XML_SetStartElementHandler
	XML_SetEndElementHandler
	XML_SetCharacterDataHandler
	XML_SetProcessingInstructionHandler
	XML_SetCommentHandler
	XML_SetCdataSectionHandler
	XML_SetStartCdataSectionHandler
	XML_SetEndCdataSectionHandler
	XML_SetDefaultHandler
	XML_SetDefaultHandlerExpand
	XML_SetDoctypeDeclHandler
	XML_SetStartDoctypeDeclHandler
	XML_SetEndDoctypeDeclHandler
	XML_SetUnparsedEntityDeclHandler
	XML_SetNotationDeclHandler
	XML_SetNamespaceDeclHandler
	XML_SetStartNamespaceDeclHandler
	XML_SetEndNamespaceDeclHandler
	XML_SetNotStandaloneHandler
	XML_SetExternalEntityRefHandler
	XML_SetExternalEntityRefHandlerArg
	XML_SetUnknownEncodingHandler
	XML_SetReturnNSTriplet
	XML_SetUserData
	XML_SetEncoding
	XML_UseParserAsHandlerArg
	XML_SetBase
	XML_GetBase
	XML_GetSpecifiedAttributeCount
	XML_GetIdAttributeIndex
	XML_Parse
	XML_GetBuffer
	XML_ParseBuffer
	XML_ExternalEntityParserCreate
	XML_SetParamEntityParsing
	XML_GetErrorCode
	XML_GetCurrentLineNumber
	XML_GetCurrentColumnNumber
	XML_GetCurrentByteIndex
	XML_GetCurrentByteCount
	XML_GetInputContext
	XML_ParserFree
	XML_ErrorString
	XML_ExpatVersion
