LIBRARY   LODBC

CODE      PRELOAD MOVEABLE DISCARDABLE
DATA      PRELOAD SINGLE

EXPORTS
	dyn_InitializeDBLink
	dyn_DeinitializeDBLink
	dyn_GetColumnsInfo
	dyn_SelectAttributes
	dyn_IsSelected
	dyn_GetSelectedAttributes
