#ifndef lint
static const char SCCSID[]="@(#)pj_list.c	4.5	93/06/12	GIE	REL";
#endif
/* Projection System: List of projections
** When adding a projection both of the following statements
** need to be updated.
*/
#include "projects.h"
	/* Prototypes */
extern void
	*pj_aea(), *pj_aeqd(), *pj_bipc(), *pj_bonne(), *pj_ob_tran(),
	*pj_cea(), *pj_eck4(), *pj_eck6(), *pj_eqc(),
	*pj_eqdc(), *pj_gnom(), *pj_laea(), *pj_lcc(),
	*pj_merc(), *pj_mill(), *pj_moll(), *pj_nsper(),
	*pj_omerc(), *pj_ortho(), *pj_poly(), *pj_sinu(),
	*pj_stere(), *pj_tmerc(), *pj_ups(), *pj_utm(), *pj_mtm(),
	*pj_vandg(), *pj_cass(), *pj_tpers(), *pj_tcea(),
	*pj_ocea(), *pj_parab(), *pj_gall(), *pj_eck5(),
	*pj_wink1(), *pj_hammer(), *pj_august(), *pj_hataea(),
	*pj_mbtfps(), *pj_mbtfpq(), *pj_putp2(), *pj_eck3(),
	*pj_mbtfpp(), *pj_putp5(), *pj_quau(), *pj_dense(),
	*pj_robin(), *pj_aitoff(), *pj_wintri(), *pj_eck1(),
	*pj_eck2(), *pj_boggs(), *pj_pconic(), *pj_rpoly(),
	*pj_airy(), *pj_bacon(), *pj_eisen(), *pj_fourn(),
	*pj_lagrng(), *pj_nicol(), *pj_ortel(), *pj_vandg2(),
	*pj_vandg3(), *pj_vandg4(), *pj_wag7(), *pj_leac(),
	*pj_loxim(), *pj_apian(), *pj_cc(), *pj_tcc(),
	*pj_collg(), *pj_goode(), *pj_chamb(), *pj_lsat(),
	*pj_alsk(), *pj_gs50(), *pj_tpeqd(), *pj_mil_os(),
	*pj_lee_os(), *pj_nzmg(), *pj_gs48(), *pj_labrd();

	struct PJ_LIST
pj_list[] = {
	"aea",		pj_aea,		"Albers Egual Area - Inv",
	"aeqd",		pj_aeqd,	"Azimuthal equidistant - Inv",
	"airy",		pj_airy,	"Airy",
	"aitoff",	pj_aitoff,	"Aitoff",
	"alsk",		pj_alsk,	"Alaska Mod.-Stereographics - Inv",
	"apian",	pj_apian,	"Apian Globular - Inv",
	"august",	pj_august,	"August Epicycloidal",
	"bacon",	pj_bacon,	"Bacon Globular",
	"bipc",		pj_bipc,	"Bipolar Conic - Inv",
	"boggs",	pj_boggs,	"Boggs Eumorphic",
	"bonne",	pj_bonne,	"Bonne - Inv",
	"cass",		pj_cass,	"Cassini - Inv",
	"cc",		pj_cc,		"Central Cylindrical - Inv",
	"cea",		pj_cea,		"Cylindrical Equal Area - Inv",
	"chamb",	pj_chamb,	"Chamberlin Trimetric",
	"collg",	pj_collg,	"Collignon - Inv",
	"dense",	pj_dense,	"Denoyer Semi-Elliptical",
	"eck1",		pj_eck1,	"Eckert I - Inv",
	"eck2",		pj_eck2,	"Eckert II - Inv",
	"eck3",		pj_eck3,	"Eckert III - Inv",
	"eck4",		pj_eck4,	"Eckert IV - Inv",
	"eck5",		pj_eck5,	"Eckert V - Inv",
	"eck6",		pj_eck6,	"Eckert VI - Inv",
	"eisen",	pj_eisen,	"Eisenlohr",
	"eqc",		pj_eqc,		"Equidistant Cylindrical - Inv",
	"eqdc",		pj_eqdc,	"Equidistant Conic - Inv",
	"fourn",	pj_fourn,	"Fournier Globular",
	"gall",		pj_gall,	"Gall (Stereographic) - Inv",
	"goode",	pj_goode,	"Goode Homolosine",
	"gnom",		pj_gnom,	"Gnomonic - Inv",
	"gs50",		pj_gs50,	"50 State U.S. Mod.-Stereographic - Inv",
	"gs48",		pj_gs48,	"48 State U.S. Mod.-Stereographic - Inv",
	"hammer",	pj_hammer,	"Hammer (Elliptical)",
	"hataea",	pj_hataea,	"Hatano Asymmetrical Equal Area - Inv",
	"labrd",	pj_labrd,	"Laborde - Inv",
	"lagrng",	pj_lagrng,	"Lagrange",
	"laea",		pj_laea,	"Lambert Azimuthal Equal Area - Inv",
	"leac",		pj_leac,	"Lambert Equal Area Conic - Inv",
	"lee_os",	pj_lee_os,	"Lee Oblate Stereographics Pacific - Inv",
	"lcc",		pj_lcc,		"Lambert Conformal Conic - Inv",
	"loxim",	pj_loxim,	"Loximuthal - Inv",
	"lsat",		pj_lsat,	"LANDSAT Space Oblique Mercator - Inv",
	"mbtfpp",	pj_mbtfpp,	"McBryde-Thomas Flat-Polar Parabolic - Inv",
	"mbtfps",	pj_mbtfps,	"McBryde-Thomas Flat-Polar Sinusoidal - Inv",
	"mbtfpq",	pj_mbtfpq,	"McBryde-Thomas Flat-Polar Quartic - Inv",
	"merc",		pj_merc,	"Mercator - Inv",
	"mill",		pj_mill,	"Miller - Inv",
	"mill_os",	pj_mil_os,	"Miller Oblate Stereographics Eur-Africa - Inv",
	"moll",		pj_moll,	"Mollweides - Inv",
	"mtm",		pj_mtm,		"Mercator Transverse Modifiee (Quebec) - Inv",
	"nicol",	pj_nicol,	"Nicolosi Globular",
	"nsper",	pj_nsper,	"General Vertical Persepective - Inv",
	"nzmg",		pj_nzmg,	"New Zealand Map Grid - Inv",
	"ob_tran",	pj_ob_tran,	"General Oblique/Transverse translator",
	"ocea",		pj_ocea,	"Oblique Cylindrical Equal Area - Inv",
	"omerc",	pj_omerc,	"Oblique Mercator - Inv",
	"ortel",	pj_ortel,	"Ortelius",
	"ortho",	pj_ortho,	"Orthographic - Inv",
	"parab",	pj_parab,	"Caster Parabolic - Inv",
	"pconic",	pj_pconic,	"Perspective Conic",
	"poly",		pj_poly,	"Polyconic (American) - Inv",
	"putp2",	pj_putp2,	"Putnins P2' - Inv",
	"putp5",	pj_putp5,	"Putnins P5 - Inv",
	"rpoly",	pj_rpoly,	"Rectangular Polyconic",
	"quau",		pj_quau,	"Quartic Authalic - Inv",
	"robin",	pj_robin,	"Robinson - Inv",
	"sinu",		pj_sinu,	"Sinusoidal - Inv",
	"stere",	pj_stere,	"Stereographic - Inv",
	"tcc",		pj_tcc,		"Transverse Central Cylindrical - Inv",
	"tcea",		pj_tcea,	"Transverse Cylindrical Equal Area - Inv",
	"tmerc",	pj_tmerc,	"Transverse Mercator - Inv",
	"tpeqd",	pj_tpeqd,	"Two Point Equidistant",
	"tpers",	pj_tpers,	"Tilted perspective - Inv",
	"ups",		pj_ups,		"Universal Polar Stereographic - Inv",
	"utm",		pj_utm,		"Universal Transverse Mercator - Inv",
	"vandg",	pj_vandg,	"Van der Grinten - Inv",
	"vandg2",	pj_vandg2,	"Van der Grinten II",
	"vandg3",	pj_vandg3,	"Van der Grinten III",
	"vandg4",	pj_vandg4,	"Van der Grinten IV",
	"wag7",		pj_wag7,	"Wagner VII",
	"wink1",	pj_wink1,	"Winkel I - Inv",
	"wintri",	pj_wintri,	"Winkel Tripel",
	0,		0,	"list terminator"
};
